#include <Python.h>
#include <QApplication>
#include <QLayout>
#include <QWidget>
#include <QMessageBox>
#include <QFileInfo>
#include <QXmlStreamReader>
#include <QMainWindow>
#include <QDesktopWidget>


/**
    This function calls the given function with the corresponding argument
    \param PyObject * : Refers to the callable function ( as a Python Object )
    \param PyObject * : Refers to the arguments to the function ( as a Python Object  )
    \return PyObject* : returns resultant of the of the called function ( as a python Object )
*/
PyObject* call_function(PyObject *callable, PyObject *args)
{
    PyObject *result = NULL, *pArgs;

    if (callable == NULL)
    {
        return NULL;
    }

    if (PyCallable_Check(callable))
    {
        if (args == NULL)
            pArgs = PyTuple_New(0);
        else
            pArgs = args;

        result = PyObject_CallObject(callable, pArgs);

        // If the arguments were created in this function then decrement
        // their reference count.
        if (args == NULL)
            Py_XDECREF(pArgs);
            // pDict and pFunc are borrowed and must not be Py_DECREF-ed.

        if (result == NULL)
        {
            PyErr_Clear();
        }
    }

    return result;
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);

    if( argc < 3 ){
        QMessageBox::warning( 0 , "Error" , "Pass Name of File as argument\n eg. <exe Name>.exe <Full FileName(.py)> <ClassName>" );
        exit(0);
    }

    QWidget *parentWidget = new QWidget;
    parentWidget->setWindowTitle( " HMI - Viewer ");

    Py_DontWriteBytecodeFlag = 1;
    Py_Initialize();

    QFileInfo objFile( argv[1] );
    QString strFileName = objFile.baseName();
    QString strFilePath = objFile.absolutePath();
    QString strClassName = argv[2];


    // Locate the path to the Python module and append it to sys.path list.
    PyRun_SimpleString("import os, sys\n");
    PyRun_SimpleString(QString("sys.path.append('"+QString(strFilePath)+"')\n").toStdString().c_str());

    PyObject *pyModule = PyImport_ImportModule(strFileName.toStdString().c_str());

    if (!pyModule) {
        QMessageBox::warning( 0 , "Error" , "Cannot create module" );
        return 1;
    }

    PyObject *pyDict = PyModule_GetDict(pyModule);
    /* pDict is a borrowed reference */

    Py_XDECREF(pyModule);


    QString strString = QString(
                QString("import PyQt5.QtGui, PyQt5.QtWidgets, sip\n"
                                 "def __embedded_factory__wrap_widget__(parentPtr):\n"
                                 "    parent = sip.wrapinstance(parentPtr, PyQt5.QtWidgets.QWidget)\n"
                                 "    return parent\n"
                                 "def __embedded_factory__create__(parent):\n"
                                 "    widget = " ) + strClassName + QString("(parent)\n"
                                 "    return sip.unwrapinstance(widget)\n" )
                );

    PyRun_String(strString.toStdString().c_str(), Py_file_input, pyDict, pyDict);


    // Get the Python module's factory function from the header file at
    // compile time.
    PyObject *pyWrapper = PyDict_GetItemString(pyDict, "__embedded_factory__wrap_widget__");
    // pyWrapper is a borrowed reference.
    PyObject *pyFactory = PyDict_GetItemString(pyDict, "__embedded_factory__create__");
    // pyFactory is a borrowed reference.

    if (!pyFactory || !pyWrapper) {
        QMessageBox::warning( 0 , "Error" , "Could Not create pyFactory or pyWrapper" );
        return 2;
    }


    // Set up the args for the Python factory function call.
    PyObject *wrapperArgs = PyTuple_New(1);
    PyObject *pyParentWidgetPtr = PyLong_FromVoidPtr(parentWidget);
    PyTuple_SetItem(wrapperArgs, 0, pyParentWidgetPtr);


    // pyParentWidgetPtr is now owned by the tuple and maintains its reference
    // (it is only referenced once because PyTuple_SetItem does not increase
    // its reference count).

    // Wrap the parent widget so that we can use it with the factory function.
    PyObject *pyParentWidget = call_function(pyWrapper, wrapperArgs);
    // Decrement the reference counts of the wrapper function and its arguments.
    Py_XDECREF(pyWrapper);
    Py_XDECREF(wrapperArgs);

    // Create a tuple containing the arguments to the factory function.
    PyObject *args = PyTuple_New(1);
    PyTuple_SetItem(args, 0, pyParentWidget);
    // pyParentWidget is now owned by the tuple and maintains its reference
    // (it is only referenced once because PyTuple_SetItem does not increase
    // its reference count).

    QGridLayout *layout = new QGridLayout(parentWidget);
    layout->setContentsMargins(0, 0, 0, 0);

    // Instantiate the class and receive a Widget pointer.
    PyObject *pyWidgetPtr = call_function(pyFactory, args);

    if (pyWidgetPtr)
    {
        // Extract the C++ pointer from the PyObject.
        QWidget *widget = (QWidget*)PyLong_AsVoidPtr(pyWidgetPtr);
        if( widget){
            //QMessageBox::warning( 0 , "Error" , "Position17- Widget Created" );
        }else{
            QMessageBox::warning( 0 , "Error" , "Position17- Widget NOT Created" );
        }
        // Decrement the reference count of the long value.
        Py_XDECREF(pyWidgetPtr);
        //QMessageBox::warning( 0 , "strFileName" , strFileName );
        layout->addWidget(widget, 0, 0);

    }
    else
    {
        QMessageBox::warning( 0 , "Error" , "Could not create Widget instance" );
        return 2;

    }

    // Decrement the references to the factory function and its arguments.
    Py_XDECREF(args);
    Py_XDECREF(pyFactory);

    QRect objRect = QApplication::desktop()->screenGeometry();
    objRect.setY(objRect.y() + 20 );
    objRect.setHeight(objRect.height() - 10 - 45 );
    parentWidget->setGeometry(objRect);
    parentWidget->show();
    int result = app.exec();

    Py_Finalize();
    return result;
}
